unit getnameu;

// Copyright  1999 by Ziff-Davis, Inc.
// Written by Neil J. Rubenking

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls;

type
  TNameForm = class(TForm)
    rbSpec      : TRadioButton;
    ebSpec      : TEdit;
    Bevel1      : TBevel;
    rbName      : TRadioButton;
    btnBrowse   : TButton;
    ebName      : TEdit;
    lblStat     : TLabel;
    btnOK       : TButton;
    btnCancel   : TButton;
    OpenDialog1 : TOpenDialog;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure rbSpecClick(Sender: TObject);
    procedure ebSpecChange(Sender: TObject);
    procedure ebSpecEnter(Sender: TObject);
    procedure ebSpecKeyPress(Sender: TObject; var Key: Char);
    procedure btnBrowseClick(Sender: TObject);
    procedure ebNameChange(Sender: TObject);
    procedure ebNameEnter(Sender: TObject);
    procedure ebNameKeyPress(Sender: TObject; var Key: Char);
  private
    { Private declarations }
    IniName : String;
    procedure CheckOK;
  public
    { Public declarations }
    procedure SetSpec(const S : String);
    function GetSpec : String;
  end;

var
  NameForm: TNameForm;

implementation

uses inifiles, allfuncs;

{$R *.DFM}

procedure TNameForm.FormCreate(Sender: TObject);
VAR S : String;
begin
  IniName := ChangeFileExt(Application.ExeName, '.INI');
  GetPosFmINI(IniName, Self, False);
  WITH TIniFile.Create(IniName) DO
  try
    S := ReadString(Self.Name + ' Settings', 'InitialDir',
      ExtractFilePath(Application.ExeName));
    IF S = '' THEN S := ExtractFilePath(Application.ExeName);
    IF S[Length(S)] <> '\' THEN S := S + '\';
    OpenDialog1.InitialDir := S;
  finally
    Free;
  end;
  lblStat.Caption := 'Must enter filename';
end;

procedure TNameForm.FormDestroy(Sender: TObject);
begin
  SetPosToINI(IniName, Self, False);
  WITH TIniFile.Create(IniName) DO
  try
    WriteString(Self.Name + ' Settings', 'InitialDir',
      OpenDialog1.InitialDir);
  finally
    Free;
  end;
end;

procedure TNameForm.FormActivate(Sender: TObject);
begin
  // Can't set the focus until form activates
  IF rbName.Checked THEN
    ebName.SetFocus
  ELSE ebSpec.SetFocus;
end;

procedure TNameForm.rbSpecClick(Sender: TObject);
begin
  IF rbSpec.Checked THEN
    ebSpecChange(ebSpec)
  ELSE ebNameChange(ebName);
end;

procedure TNameForm.ebSpecChange(Sender: TObject);
VAR
  Sn, Se : String;
  N : Integer;
begin
  // Deal with pasted-in text that contains invalid chars
  Sn := (Sender AS TEdit).Text;
  FOR N := length(Sn) DOWNTO 1 DO
    CASE Sn[N] OF
      '"',':','<','>','|','\','/' : Delete(Sn, N, 1);
    END;
  IF Sn <> (Sender AS TEdit).Text THEN
    begin
      (Sender AS TEdit).Text     := Sn;
      (Sender AS TEdit).SelStart := length(Sn);
    end;
  N := LastDelimiter('.', Sn);
  IF N = 0 THEN Se := ''
  ELSE
    begin
      Se := Copy(Sn, N+1, Length(Sn)-N);
      SetLength(Sn, N-1);
    end;
  IF Sn = '' THEN
    lblStat.Caption := 'Must enter filename'
  ELSE IF Pos('.', (Sender AS TEdit).Text) = 0 THEN
    lblStat.Caption := 'Must enter extension (or period)'
  ELSE IF Se = '' THEN
    lblStat.Caption := 'Note: Only matches files with no extension'
  ELSE IF Length(Se) > 3 THEN
    lblStat.Caption := 'Note: Extension longer than 3 letters'
  ELSE IF (Se = '*') AND (Sn = '*') THEN
    lblStat.Caption := 'Warning: Matches all files!'
  ELSE IF (pos('*', Se) > 0) AND
          (pos('*', Sn) > 0) THEN
    lblStat.Caption := 'Warning: * in both name and extension'
  ELSE IF (pos('*', Se)  = 0) AND
          (pos('*', Sn) = 0) AND
          (pos('?', Se)  = 0) AND
          (pos('?', Sn) = 0) THEN
    lblStat.Caption := Format('Note: Matches exact filename '+
      'only', [Sn, Se])
  ELSE lblStat.Caption := '';
  CheckOK;
end;

procedure TNameForm.ebSpecEnter(Sender: TObject);
begin
  rbSpec.Checked := True;
end;

procedure TNameForm.ebSpecKeyPress(Sender: TObject; var Key: Char);
begin
  CASE Key OF
    // Don't accept chars that are invalid in a filespec
    '"',':','<','>','|','\','/' : Key := #0;
  END;
end;

procedure TNameForm.btnBrowseClick(Sender: TObject);
begin
  WITH OpenDialog1 DO
    IF Execute THEN
      begin
        rbName.Checked := True;
        ebName.Text := Filename;
      end;
end;

procedure TNameForm.ebNameChange(Sender: TObject);
VAR
  S : String;
  N : Integer;
begin
  // Deal with pasted-in text that contains invalid chars
  S := (Sender AS TEdit).Text;
  FOR N := length(S) DOWNTO 1 DO
    CASE S[N] OF
      '"','<','>','|','?','*' : Delete(S, N, 1);
    END;
  IF S <> (Sender AS TEdit).Text THEN
    begin
      (Sender AS TEdit).Text     := S;
      (Sender AS TEdit).SelStart := length(S);
    end;
  IF Pos(':\', S) <> 2 THEN
    lblStat.Caption := 'Enter full path'
  ELSE IF FileExists(S) THEN
    lblStat.Caption := 'File exists'
  ELSE lblStat.Caption := 'File does not exist';
  CheckOK;
end;

procedure TNameForm.ebNameEnter(Sender: TObject);
begin
  rbName.Checked := True;
end;

procedure TNameForm.ebNameKeyPress(Sender: TObject; var Key: Char);
begin
  CASE Key OF
    // Don't accept chars that are invalid in a filename
    '"','<','>','|','*','?' : Key := #0;
  END;
end;

function TNameForm.GetSpec: String;
begin
  IF rbSpec.Checked THEN
    Result := ebSpec.Text
  ELSE Result := ebName.Text;
end;

procedure TNameForm.SetSpec(const S: String);
begin
  IF Pos(':', S) > 0 THEN
    begin
      rbName.Checked := True;
      ebName.Text    := S;
    end
  ELSE
    begin
      rbSpec.Checked := True;
      ebSpec.Text    := S;
    end;
end;

procedure TNameForm.CheckOK;
begin
  IF rbName.Checked THEN
    btnOK.Enabled := Pos(':\', ebName.Text) = 2
  ELSE
    btnOK.Enabled := (Pos('.', ebSpec.Text) > 1) AND
      (Length(ebSpec.Text) > 1);
end;

end.
